// SMTP.cpp: implementation of the CSMTP class.
// Copyright (c) 1998, Wes Clyburn
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SMTP.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void ODS(PCWSTR str);
void ODS1(PCWSTR str, DWORD parm);
void ODS2(PCWSTR str, DWORD parm1, DWORD parm2);

// Static member initializers
//

// Note: the order of the entries is important.
//       They must be synchronized with eResponse entries. 
CSMTP::response_code CSMTP::response_table[] =
{
	{ 250, TEXT("SMTP server error") },								// GENERIC_SUCCESS
	{ 220, TEXT("SMTP server not available") },					// CONNECT_SUCCESS
	{ 354, TEXT("SMTP server not ready for data") },			// DATA_SUCCESS
	{ 221, TEXT("SMTP server didn't terminate session") }  	// QUIT_SUCCESS
};

void ODS(PCWSTR str)
{
	OutputDebugString(str);
	Sleep(10);
}

void ODS1(PCWSTR str, DWORD parm)
{
	WCHAR str2[100];
	wsprintf(str2, str, parm);
	ODS(str2);
}

void ODS2(PCWSTR str, DWORD parm1, DWORD parm2)
{
	WCHAR str2[100];
	wsprintf(str2, str, parm1, parm2);
	ODS(str2);
}

PCWSTR ToWide(LPCSTR pIn)
{
	static WCHAR Out[100];
	mbstowcs(Out, pIn, strlen(pIn));
	return Out;
}

LPCSTR ToNarrow(PCWSTR pIn)
{
	static char Out[100];
	wcstombs(Out, pIn, wcslen(pIn));
	return Out;
}

int InitAddr(struct sockaddr_in *remote_addr, PCWSTR remote_host_name, WORD Port) 
{
	int remote_addr_len;
	struct hostent *remote_host;

	remote_host = gethostbyname(ToNarrow(remote_host_name));
	if(!remote_host) 
		{
		OutputDebugString(TEXT("socket_remote: ERROR! Could not get host information."));
		return 1;  
		}

	remote_addr_len = sizeof(*remote_addr);
	memset(remote_addr, 0, remote_addr_len);
	remote_addr->sin_family = AF_INET;
	remote_addr->sin_port = htons(Port);
	remote_addr->sin_addr = *(struct in_addr * )(remote_host->h_addr_list[0]);
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSMTP::CSMTP( LPCTSTR szSMTPServerName, UINT nPort )
{
	ASSERT( szSMTPServerName != NULL );
//	AfxSocketInit();
	m_sMailerName = TEXT("WC Mail");
	m_sSMTPServerHostName = szSMTPServerName;
	m_nPort = nPort;
	m_bConnected = FALSE;
	m_sError = TEXT("OK");
}

CSMTP::~CSMTP()
{
	if(m_bConnected)
		Disconnect();
}

CStr CSMTP::GetServerHostName()
{
	return m_sSMTPServerHostName;
}

BOOL CSMTP::Connect()
{
	CStr sHello;
	if(m_bConnected)
		return TRUE;

	m_wsSMTPServer = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	
	if(m_wsSMTPServer == INVALID_SOCKET)
		{
		m_sError = TEXT("Unable to create the socket.");
		return FALSE;
		}

	sockaddr_in addr;

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = 0;
	addr.sin_addr.S_un.S_addr = INADDR_ANY;

	if(bind(m_wsSMTPServer, (const struct sockaddr *)&addr, sizeof(addr)))
		{
		ODS1(TEXT("Error %d binding to server"), ::GetLastError());
		m_sError = TEXT( "Unable to bind to server" );
		closesocket(m_wsSMTPServer);
		return FALSE;
		}

//	memset(&addr, 0, sizeof(addr));
//	addr.sin_family = AF_INET;
//	addr.sin_port = htons(25);
//	addr.sin_addr.S_un.S_un_b.s_b1 = 192;
//	addr.sin_addr.S_un.S_un_b.s_b2 = 168;
//	addr.sin_addr.S_un.S_un_b.s_b3 = 0;
//	addr.sin_addr.S_un.S_un_b.s_b4 = 1;

	InitAddr(&addr, GetServerHostName(), GetPort());

	if(connect(m_wsSMTPServer, (const struct sockaddr *)&addr, sizeof(addr)) == SOCKET_ERROR)
		{
		ODS1(TEXT("Error %d connecting to server"), ::GetLastError());
		m_sError = TEXT("Unable to connect to server");
		closesocket(m_wsSMTPServer);
		return FALSE;
		}

	if(!get_response(CONNECT_SUCCESS))
		{
		m_sError = TEXT( "Server didn't respond." );
		closesocket(m_wsSMTPServer);
		return FALSE;
		}

	char LocalName[80];
	gethostname(LocalName, 80);
	sHello.Format(L"HELO %s\r\n", ToWide(LocalName));

	send(m_wsSMTPServer, ToNarrow(sHello), sHello.GetLength(), 0);
	if(!get_response(GENERIC_SUCCESS))
		{
		closesocket(m_wsSMTPServer);
		return FALSE;
		}

	m_bConnected = TRUE;
	return TRUE;
}

BOOL CSMTP::Disconnect()
{
	BOOL ret;
	if(!m_bConnected)
		return TRUE;

	// Disconnect gracefully from the server and close the socket
	CStr sQuit = TEXT("QUIT\r\n");
	send(m_wsSMTPServer, ToNarrow(sQuit), sQuit.GetLength(), 0);

	// No need to check return value here.
	// If it fails, the message is available with GetLastError
	ret = get_response(QUIT_SUCCESS);
	closesocket(m_wsSMTPServer);

	m_bConnected = FALSE;
	return ret;
}

UINT CSMTP::GetPort()
{
	return m_nPort;
}

CStr CSMTP::GetMailerName()
{
	return m_sMailerName;
}

CStr CSMTP::GetLastError()
{
	return m_sError;
}

BOOL CSMTP::SendMessage(CMailMessage *msg)
{
	ASSERT(msg != NULL);
	if(!m_bConnected)
		{
		m_sError = TEXT("Must be connected");
		return FALSE;
		}

	if(FormatMailMessage(msg) == FALSE)
		{
		return FALSE;
		}

	if(transmit_message(msg) == FALSE)
		{
		return FALSE;
		}

	return TRUE;
}

BOOL CSMTP::FormatMailMessage(CMailMessage *msg)
{
	ASSERT(msg != NULL);
	if(prepare_header(msg) == FALSE)
		{
		return FALSE;
		}

	// Append a CR/LF to body if necessary.
	CStr temp;
	msg->m_sBody.GetRight(2, temp);
	if(temp != TEXT("\r\n"))
		msg->m_sBody += TEXT("\r\n");
	return TRUE;
}

void CSMTP::SetServerProperties(LPCTSTR sServerHostName, UINT nPort)
{
	ASSERT(sServerHostName != NULL);
	// Needs to be safe in non-debug too
	if( sServerHostName == NULL )
		return;

	m_sSMTPServerHostName = sServerHostName;
	m_nPort = nPort;
}

// Create the SMTP message header as per RFC822
BOOL CSMTP::prepare_header(CMailMessage * msg)
{
	ASSERT(msg != NULL);
	CStr sTo;
	CStr sDate;
	CStr& sHeader = msg->m_sHeader;	// Shorthand

	if(msg->GetNumRecipients() <= 0)
		{
		m_sError = TEXT("No Recipients");
		return FALSE;
		}

	sHeader = TEXT("");	// Clear it

	// Get the recipients into a single string
	sTo = TEXT("");
	CStr sEmail = TEXT("");
	CStr sFriendly = TEXT("");
	for(int i = 0; i < msg->GetNumRecipients(); i++)
		{
		msg->GetRecipient(sEmail, sFriendly, i);
		sTo += (i > 0 ? TEXT(",") : TEXT(""));
		sTo += sFriendly;
		sTo += TEXT("<");
		sTo += sEmail;
		sTo += TEXT(">");
	}

//	msg->m_tDateTime = msg->m_tDateTime.GetCurrentTime();
	// Format: Mon, 01 Jun 98 01:10:30 GMT
//	sDate = msg->m_tDateTime.Format( "%a, %d %b %y %H:%M:%S %Z" );
	sDate = TEXT("xxxx");
	sHeader.Format(TEXT("Date: %s\r\nFrom: %s\r\nTo: %s\r\nSubject: %s\r\nX-Mailer: <%s>\r\n\r\n"),
					// Include other extension lines if desired
					(LPCTSTR)sDate,
					(LPCTSTR)msg->m_sFrom,
					(LPCTSTR)sTo,
					(LPCTSTR)msg->m_sSubject,
					(LPCTSTR)m_sMailerName );
	return TRUE;
}

CStr CSMTP::prepare_body(CMailMessage *msg)
{
	ASSERT(msg != NULL);
	CStr sTemp;
	CStr sCooked = TEXT("");
	LPTSTR szBad = TEXT("\r\n.\r\n");
	LPTSTR szGood = TEXT("\r\n..\r\n");
	int nPos;
	int nStart = 0;
	int nBadLength = wcslen(szBad);
	sTemp = msg->m_sBody;
	
	CStr temp;
	sTemp.GetLeft(3, temp);
	if(temp == TEXT(".\r\n"))
		sTemp = TEXT(".") + sTemp;
	//
	// This is a little inefficient because it beings a search
	// at the beginning of the string each time. This was
	// the only thing I could think of that handled ALL variations.
	// In particular, the sequence "\r\n.\r\n.\r\n" is troublesome. 
	// (Even CStringEx's FindReplace wouldn't handle that situation
	// with the global flag set.)
	//

	while((nPos = sTemp.Find( szBad )) > -1 )
		{
		sTemp.GetMiddle(nStart, nPos, sCooked);
		sCooked += szGood;
		sTemp.GetRight( sTemp.GetLength() - (nPos + nBadLength), temp);
		sTemp = sCooked + temp;
		}

	return sTemp;
}

BOOL CSMTP::transmit_message(CMailMessage *msg)
{
	CStr sFrom;
	CStr sTo;
	CStr sTemp;
	CStr sEmail;

	ASSERT(msg != NULL);
	if(!m_bConnected)
		{
		m_sError = TEXT("Must be connected");
		return FALSE;
		}

	// Send the MAIL command
	//
	sFrom.Format(TEXT("MAIL From: <%s>\r\n"), (LPCTSTR)msg->m_sFrom);

	send(m_wsSMTPServer, ToNarrow(sFrom), sFrom.GetLength(), 0);
	if(!get_response(GENERIC_SUCCESS))
		return FALSE;
	
	// Send RCPT commands (one for each recipient)
	//
	for(int i = 0; i < msg->GetNumRecipients(); i++)
		{
		msg->GetRecipient(sEmail, sTemp, i);
		sTo.Format( TEXT("RCPT TO: <%s>\r\n"), (LPCTSTR)sEmail);
		send(m_wsSMTPServer, ToNarrow(sTo), sTo.GetLength(), 0);
		get_response(GENERIC_SUCCESS);
		}

	// Send the DATA command
	sTemp = TEXT("DATA\r\n");
	send(m_wsSMTPServer, ToNarrow(sTemp), sTemp.GetLength(), 0);
	if(!get_response( DATA_SUCCESS))
		{
		return FALSE;
		}

	// Send the header
	//
	send(m_wsSMTPServer, ToNarrow(msg->m_sHeader), msg->m_sHeader.GetLength(), 0);

	// Send the body
	//
	sTemp = prepare_body(msg);
	send(m_wsSMTPServer, ToNarrow(sTemp), sTemp.GetLength(), 0);

	// Signal end of data
	//
	sTemp = TEXT("\r\n.\r\n");
	send(m_wsSMTPServer, ToNarrow(sTemp), sTemp.GetLength(), 0);
	if(!get_response(GENERIC_SUCCESS))
		{
		return FALSE;
		}

	return TRUE;
}

BOOL CSMTP::get_response(UINT response_expected)
{
	ASSERT(response_expected >= GENERIC_SUCCESS);
	ASSERT(response_expected < LAST_RESPONSE);

	CStr sResponse;
	UINT response;
	response_code* pResp;	// Shorthand

	char buff[1024];
	
	if(recv(m_wsSMTPServer, buff, sizeof(buff), 0) == SOCKET_ERROR)
		{
		m_sError = TEXT("Socket Error");
		return FALSE;
		}

	sResponse = ToWide(buff);

	CStr temp;
	sResponse.GetLeft(3, temp);
	swscanf((LPCTSTR)temp, TEXT("%d"), &response);
	pResp = &response_table[response_expected];
	if(response != pResp->nResponse)
		{
		m_sError.Format(TEXT("%d:%s"), response, (LPCTSTR)pResp->sMessage);
		return FALSE;
		}

	return TRUE;
}


