// SerialIO.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include <iobase.h>

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	RETAILMSG( 1, ( TEXT( "Serial IO started!" ) ) );

	// Set thread priority to highest. This dramatically improves scan consistency.
	//
	SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL );

	// Create and initialize the I/O base interface
	//
	CIOBase IOB;
	IOB.Init();

	// Turn on the run mode LED
	//
	IOB.TurnOnRunLED();	

	// Open the PORT for reading and writing
	//
	HANDLE g_hSerial= CreateFile( TEXT("COM1:"), 
						   GENERIC_READ | GENERIC_WRITE, 
						   0, 
						   NULL, 
						   OPEN_EXISTING, 
						   FILE_ATTRIBUTE_NORMAL, 
						   NULL );
 
    if( g_hSerial== INVALID_HANDLE_VALUE ) 
	{
		RETAILMSG( 1, ( TEXT( " CommPort : Failed to open. Error %d\r\n"), GetLastError() ) );
	
	}else
	{
		DCB  dcb;

		BOOL bSuccess= GetCommState( g_hSerial, (LPDCB)&dcb );
 
		if( !bSuccess ) 
		{
			RETAILMSG( 1, (TEXT( " CommPort: GetCommState error %d\r\n" ), GetLastError()));
		
			CloseHandle( g_hSerial );
	
		}else
		{
			const	DWORD	cBaudRate= 19200;
				
			BYTE	cParity  = NOPARITY,	// No parity
					cByteSize= DATABITS_8,	// 8 bits per byte
					cStopBits= 0;			// 1 stop bit
			
			dcb.BaudRate= cBaudRate;
			dcb.Parity  = cParity;
			dcb.ByteSize= cByteSize;
			dcb.StopBits= cStopBits;
 
			if( !SetCommState( g_hSerial, (LPDCB)&dcb ) ) 
			{
				RETAILMSG(1, ( TEXT( " CommPort : SetCommState error %d\r\n" ), GetLastError() ) );
		
				CloseHandle(g_hSerial);
		
			}else
			{
				COMMTIMEOUTS	tos;
 
				tos.ReadIntervalTimeout=		10;
				tos.ReadTotalTimeoutConstant=	100;
				tos.ReadTotalTimeoutMultiplier= 10;
				tos.WriteTotalTimeoutConstant=	100;
				tos.WriteTotalTimeoutMultiplier=10;
 
				SetCommTimeouts(g_hSerial, &tos);
 
				RETAILMSG(1, ( TEXT( "\tListening on COM1:" ) ) );
 
				// Echo back anything that is received
				//
				BYTE BufferIn[600];
				
				DWORD NumIn, NumWritten;

				while( 1 )
				{
				   int Num= ReadFile( g_hSerial, BufferIn, 600, &NumIn, NULL );
 
					if(NumIn)
					{
						WriteFile( g_hSerial, BufferIn, NumIn, &NumWritten, NULL );
 
						if( NumWritten != NumIn )
						{
							RETAILMSG( 1, ( TEXT( "\t\tError: NumRead: %d  NumWritten: %d" ), NumIn, NumWritten ) );
						}
					}

 					// Give the WinPLC a chance to do other stuff if it needs to
					//
					Sleep( 5 );
				}
			}
		}
	}

	// Turn off the run mode LED
	//
	IOB.TurnOffRunLED();
	
	RETAILMSG( 1, ( TEXT( "Serial IO exiting. Bye!" ) ) );

	return 0;
}

