// IOBase.h : Declaration of the CIOBase

#ifndef __IOBASE_H_
#define __IOBASE_H_

#include <winplcrt.h>

class CIOBase;
class CModule;

class CModule
	{
	friend CIOBase;

	public:
		CModule(BYTE Type, BYTE ID, BYTE X, BYTE Y, BYTE WX, BYTE WY, BYTE DWX, BYTE DWY);

		int WriteModuleState(BYTE *pBuff);

		void SetXState(void *pX, int Len){ memcpy(X, pX, Len); }
		void SetYState(void *pY, int Len){ memcpy(Y, pY, Len); }
		void SetWXState(void *pWX, int Len){ memcpy(WX, pWX, Len); }
		void SetWYState(void *pWY, int Len){ memcpy(WY, pWY, Len); }
		void SetDWXState(void *pDWX, int Len){ memcpy(DWX, pDWX, Len); }
		void SetDWYState(void *pDWY, int Len){ memcpy(DWY, pDWY, Len); }

		int GetXState(void *pX){ memcpy(pX, X, XCount/8); return XCount/8; }
		int GetYState(void *pY){ memcpy(pY, Y, YCount/8); return YCount/8; }
		int GetWXState(void *pWX){ memcpy(pWX, WX, WXCount*2); return WXCount*2; }
		int GetWYState(void *pWY){ memcpy(pWY, WY, WYCount*2); return WYCount*2; }
		int GetDWXState(void *pDWX){ memcpy(pDWX, DWX, DWXCount*4); return DWXCount*4; }
		int GetDWYState(void *pDWY){ memcpy(pDWY, DWY, DWYCount*4); return DWYCount*4; }
		
		BOOL IsIn(){ return XCount || WXCount || DWXCount; }
		BOOL IsOut(){ return YCount || WYCount || DWYCount; }
		BOOL IsDiscrete(){ return XCount || YCount; }
		BOOL IsAnalog(){ return WXCount || WYCount; }
		BOOL IsDouble(){ return DWXCount || DWYCount; }
		BOOL GetX(int BitNum){ return X[BitNum/8] & (0x01<<(BitNum%8)); }
		BOOL GetY(int BitNum){ return Y[BitNum/8] & (0x01<<(BitNum%8)); }
		void SetY(int BitNum, BOOL Val){ Y[BitNum/8] = Y[BitNum/8] & ~(0x01<<(BitNum%8)) | (Val ? (0x01<<(BitNum%8)) : 0); }

		void ClearOutputs();

		int GetXCount(){ return XCount; }
		int GetYCount(){ return YCount; }
		int GetWXCount(){ return WXCount; }
		int GetWYCount(){ return WYCount; }
		int GetDWXCount(){ return DWXCount; }
		int GetDWYCount(){ return DWYCount; }

	private:
		BYTE ModuleType;
		BYTE ModuleID;
		BYTE XCount;
		BYTE YCount;
		BYTE WXCount;
		BYTE WYCount;
		BYTE DWXCount;
		BYTE DWYCount;

		BYTE X[32];
		BYTE Y[32];
		WORD WX[32];
		WORD WY[32];
		DWORD DWX[16];
		DWORD DWY[16];
	};

class CIOBase
{
	public:
		CIOBase();

		/*
		** Initialization
		*/
		int Init();
		int Exit();
		int OnIdle(int SleepTime=1, int PollCount=0, BOOL ClearScanCount=TRUE);

		/*
		** IO driver management
		*/
		BOOL LockIOWrites();
		BOOL UnlockIOWrites();
		int ConfigIOScan(DWORD ScanPriority, DWORD SleepTime);
		int PollIO();
		int GetIOScanCount(BOOL Reset=TRUE);

		/*
		** Image register functions
		*/
		int WriteIO();
		int ReadIO();

#ifdef DEBUG_IO
		// Get/set bits by slot:point
		BOOL GetDI(short Slot, short Point);
		BOOL GetDO(short Slot, short Point);
		void SetDO(short Slot, short Point, BOOL NewVal);

		// Get/set words by slot:point
		WORD GetWI(short Slot, short Point);
		WORD GetWO(short Slot, short Point);
		void SetWO(short Slot, short Point, WORD NewVal);

		// Get/set dwords by slot:point
		DWORD GetDWI(short Slot, short Point);
		DWORD GetDWO(short Slot, short Point);
		void SetDWO(short Slot, short Point, DWORD NewVal);
#else
		// Get/set bits by slot:point
		BOOL GetDI(short Slot, short Point){ return aModules[Slot]->GetX(Point); }
		BOOL GetDO(short Slot, short Point){ return aModules[Slot]->GetY(Point); }
		void SetDO(short Slot, short Point, BOOL NewVal){ aModules[Slot]->SetY(Point, NewVal); }

		// Get/set words by slot:point
		WORD GetWI(short Slot, short Point){ return aModules[Slot]->WX[Point]; }
		WORD GetWO(short Slot, short Point){ return aModules[Slot]->WY[Point]; }
		void SetWO(short Slot, short Point, WORD NewVal){ aModules[Slot]->WY[Point] = NewVal; }

		// Get/set dwords by slot:point
		DWORD GetDWI(short Slot, short Point){ return aModules[Slot]->DWX[Point]; }
		DWORD GetDWO(short Slot, short Point){ return aModules[Slot]->DWY[Point]; }
		void SetDWO(short Slot, short Point, DWORD NewVal){ aModules[Slot]->DWY[Point] = NewVal; }
#endif

		// Get/set bit by PLC style point
		BOOL GetX(short Point);
		BOOL GetY(short Point);
		BOOL SetY(short Point, BOOL NewVal);

		// Get/set words by PLC style point
		WORD GetWX(short Point);
		WORD GetWY(short Point);
		BOOL SetWY(short Point, WORD NewVal);

		// Get/set dwords by PLC style point
		DWORD GetDWX(short Point);
		DWORD GetDWY(short Point);
		BOOL SetDWY(short Point, DWORD NewVal);

		// Clear all outputs
		void ClearOutputs();
		
		/*
		** Utility
		*/
		void OutputDebugString(LPCSTR pStr);
		void TurnOnRunLED();
		void TurnOffRunLED();

		/*
		** Module access
		*/
		int GetModuleCount(){ return sizeof(aModules)/sizeof(aModules[0]); }
		CModule *GetModule(int ModNum){ return aModules[ModNum]; }

	protected:
		void ParseInputPacket(int Length);
		int PackOutputPacket();
		CModule *aModules[8];

		WORD aXInSlot[8];
		WORD aYInSlot[8];
		WORD aWXInSlot[8];
		WORD aWYInSlot[8];
		WORD aDWXInSlot[8];
		WORD aDWYInSlot[8];

		BYTE InBuff[200];
		BYTE OutBuff[200];
		WPLCInfo Info;
};

#endif //__IOBASE_H_
