#include "ProtocolHandler.h"

class CIOBase;

#define VSIZE	010000
#define CSIZE	04000

#define SOH		0x01
#define STX		0x02
#define ETX		0x03
#define EOT		0x04
#define ENQ		0x05
#define ACK		0x06
#define NAK		0x15
#define ETB		0x17

#define KSQ_SUCCESS	-1
#define KSQ_FAILURE	0

struct PacketInfo
{
	sockaddr_in Addr;
	WORD PacketNum;
};

class KSeqHandler : public ProtocolHandler
{
	public:
		KSeqHandler(BOOL ExposeIO);
		virtual ~KSeqHandler();

		virtual BOOL InitSerialHandler(WCHAR *pPort, DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits);
		virtual BOOL InitEthernetHandler(WORD Port);
		virtual BOOL ExitHandler();

	protected:
		virtual void ProvideTC2AData(BYTE *pData, WORD Address, WORD Count);
		virtual void ProvideTC50Data(BYTE *pData, WORD Address, WORD Count);
		virtual void ProvideTC40Data(BYTE *pData, WORD Address, WORD Count);

	private:
		virtual DWORD SerialProc();
		virtual DWORD EtherProc();

		int RxEnqMsg();
		void TxAckMsg();
		int RxHeader();
		int RxControl();
		void TxData();
		void TxControl(BYTE Which);
		void BuildTC51Data();
		void BuildTC50Data(WORD Address, WORD Count);
		void BuildTC46Data();
		void BuildTC40Data(WORD Address, WORD Count);
		void BuildTC2AData(WORD Address, WORD Count);
		void BuildTC44Data();
		void BuildTC45Data();
		int ValidateHeader();
		void BuildResponse();
		BYTE CalcLRC(BYTE *pData, int Count);

		BOOL ValidateHAPPacket(BYTE *pBuff, int Length, PacketInfo *pPI);
		int SendEtherAck(PacketInfo *pPI, BYTE Function, WORD Warning);
		int SendEtherNak(PacketInfo *pPI, BYTE Function, WORD Error);
		WORD BuildHAPPacket(BYTE *pBuff, PacketInfo *pPI, BYTE *pData, WORD DataLen);
		int ProcessKSeqRequest(BYTE *pBuff, int Length, PacketInfo *pPI);
		int ProcessCCMRequest(BYTE *pBuff, PacketInfo *pPI);
		int ProcessReadDeviceInfo(PacketInfo *pPI);
		void UpdateIOBuffers();
		void ProvideBitMemory(BYTE *pData, BYTE *pSource, WORD Address, WORD Count);
		void PadBitData(BYTE *pData, WORD StartBit, WORD EndBit);

		BYTE State;
		BYTE SubState;
		BYTE CommBuff[256];
		WORD CBCount;
		WORD BytesLeft;
		BYTE ErrorCode;
		BYTE TaskCode;
		BYTE Retries;
		DWORD StartTime;
		WORD TCAddress;
		WORD TCCount;
		BYTE CommResult;
		CIOBase *pIOB;
		BYTE XBuff[256];
		BYTE YBuff[256];
		BYTE IOBuff[2048];
};

/*
** *** CommResult ***
**
** Codes returned by ProcessDVComm to the application.
*/
enum CommResult
	{
	crInactive,					/* No comm pending, nothing to do		*/
	crWaiting,					/* Comm in process, waiting for input	*/
	crCompletedMessagePoll,	/* Completed message data write			*/
	crCompletedNumericPoll,	/* Completed numeric data write			*/
	crCompletedKeyPress,		/* Completed key state read				*/
	crCompletedMisc,			/* Completed unspecified transaction	*/
	crCommError,				/* Comm error occured						*/
	};

/*
** *** CommState ***
**
** State values for comm state machine
*/
enum CommState
	{
	csInit,
	csIdle,
	csRxEnqMsg,
	csRxHeader,
	csTxData,
	csRxAck,
	csRxEot,
	};

/*
** *** CommSubState ***
**
** Sub state values for comm state machine
*/
enum CommSubState
	{
	cssInit,
	cssWaiting,
	};
	
/*
** *** CommStateResult ***
**
** State execution results
*/
enum CommStateResult
	{
	csrWaiting,
	csrCompleted,
	csrFailed,
	csrUnsupportedTaskCode,
	csrRxNak,
	csrRxAck,
	csrRxEot,
	};

/*
** *** CommError ***
**
** Comm error codes
*/
enum CommError
	{
	ceNoError = 0,
	ceTimeout,
	ceDataError,
	ceUnsupportedTaskCode,
	ceNAKedOut,
	ceRangeError,
	ceUnsupportedDataType,
	};

